/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef IFONT_H
#define IFONT_H

#include <Arduino.h>

class IFont {

  public:

    /**
      Returns the maximum character width.
    */
    virtual uint8_t getCharWidth() = 0;

    /**
      Returns the character width for a specified character.
      For monospaced fonts the returned width will always be the same, regardless of character.
    */
    virtual uint8_t getCharWidth(char c) = 0;

    /**
      Returns the character height. Currently only heights not exceeding 8 are supported.
    */
    virtual uint8_t getCharHeight() = 0;

    /**
      Returns the character column data.
    */
    virtual char getColumnData(char c, uint8_t column) = 0;

    /**
      How many pixels of extra spacing that should be added in between characters.
    */
    virtual uint8_t getSpacingBetweenChars() = 0;
};

#endif // IFONT_H
