/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "PimientoButtons.h"

PimientoButtons buttons;

void PimientoButtons::init(uint8_t buttonPin0) {
  init(buttonPin0, 0, 0, 0, 0);
}

void PimientoButtons::init(uint8_t buttonPin0, uint8_t buttonPin1) {
  init(buttonPin0, buttonPin1, 0, 0, 0);
}

void PimientoButtons::init(uint8_t buttonPin0, uint8_t buttonPin1, uint8_t buttonPin2) {
  init(buttonPin0, buttonPin1, buttonPin2, 0, 0);
}

void PimientoButtons::init(uint8_t buttonPin0, uint8_t buttonPin1, uint8_t buttonPin2, uint8_t buttonPin3) {
  init(buttonPin0, buttonPin1, buttonPin2, buttonPin3, 0);
}

void PimientoButtons::init(uint8_t buttonPin0, uint8_t buttonPin1, uint8_t buttonPin2, uint8_t buttonPin3, uint8_t buttonPin4) {

  buttonPins[0] = buttonPin0;
  buttonPins[1] = buttonPin1;
  buttonPins[2] = buttonPin2;
  buttonPins[3] = buttonPin3;
  buttonPins[4] = buttonPin4;

  for (uint8_t i = 0; i < BUTTON_COUNT; i++) {
    if (isConfiguredButton(i)) {
      clearState(i);
      pinMode(buttonPins[i], INPUT);
    }
  }
}

int8_t PimientoButtons::readStates() {

  for (uint8_t i = 0; i < BUTTON_COUNT; i++) {

    if (isConfiguredButton(i)) {

      uint8_t pin = buttonPins[i];

      if (buttonEvent[i] == EVENT_NONE) {

        int newButtonState = digitalRead(pin);

        if (newButtonState != oldButtonState[i]) {

          if (newButtonState == BUTTON_STATE_NOT_PRESSED && oldButtonState[i] == BUTTON_STATE_PRESSED) {
            buttonEvent[i] = EVENT_RELEASED;
          } else if (newButtonState == BUTTON_STATE_PRESSED && oldButtonState[i] == BUTTON_STATE_NOT_PRESSED) {
            buttonEvent[i] = EVENT_PRESSED;
          }

          oldButtonState[i] = newButtonState;
          return i;
        }
      }
    }
  }
  return -1;
}

uint8_t PimientoButtons::consumeEvent(uint8_t buttonIndex) {
  if (!isConfiguredButton(buttonIndex)) {
    return EVENT_NONE;
  }
  uint8_t result = getEvent(buttonIndex);
  if (result != EVENT_NONE) {
    oldButtonEvent[buttonIndex] = result;
    buttonEvent[buttonIndex] = EVENT_NONE;
  }
  return result;
}

uint8_t PimientoButtons::getEvent(uint8_t buttonIndex) {
  if (!isConfiguredButton(buttonIndex)) {
    Serial.println("getEvent not configured button");
    return EVENT_NONE;
  }
  return buttonEvent[buttonIndex];
}

uint8_t PimientoButtons::clearState(uint8_t buttonIndex) {
  if (isConfiguredButton(buttonIndex)) {
    buttonEvent[buttonIndex] = EVENT_NONE;
    oldButtonState[buttonIndex] = BUTTON_STATE_NOT_PRESSED;
    oldButtonEvent[buttonIndex] = EVENT_NONE;
  }
}

bool PimientoButtons::isConfiguredButton(uint8_t buttonIndex) {
  if (buttonIndex < 0 || buttonIndex >= BUTTON_COUNT) {
    return false;
  }
  return (buttonPins[buttonIndex] != 0);
}
