/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "PimientoScheduler.h"

void PimientoScheduler::until(const boolean exitCondition, const uint32_t delay, uint32_t &counter, Callback c) {
  if (!exitCondition) {
    everyMillis(delay, counter, c);
  }
}

void PimientoScheduler::once(uint8_t &state, Callback c) {
  if (state == 0) {
    c();
    state = 1;
  }
}

void PimientoScheduler::repeat(uint32_t delay, uint32_t &delayCounter, uint16_t &timesCounter, Callback c) {
  if (timesCounter != 0) {
    if (timesCounter > 0) {
      everyMillis(delay, delayCounter, c);
      timesCounter--;
    }
  }
}

void PimientoScheduler::everyMillis(const uint32_t delay, uint32_t &counter, Callback c) {
  if (millis() - counter >= delay) {
    c();
    counter = millis();
  }
}
