/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef PIMIENTO_SCHEDULER_H
#define PIMIENTO_SCHEDULER_H

#include <Arduino.h>

typedef void (*Callback)(void);

class PimientoScheduler {

  public:

    /** Runs the specified parameterless function <Callback> until the <exitCondition> is true.
        The <counter> parameter is used to maintain state.
    **/
    static void until(const boolean exitCondition, const uint32_t delay, uint32_t &counter, Callback c);

    /** Runs the specified parameterless function <Callback> once.
        The <state> parameter is used to maintain state.
    **/
    static void once(uint8_t &state, Callback c);

    /** Runs the specified parameterless function <Callback> <timesCounter> times, every <delay> milliseconds.
        The <delayCounter> and <timesCounter> parameters are used for book-keeping and are updated on every call.
    */
    static void repeat(uint32_t delay, uint32_t &delayCounter, uint16_t &timesCounter, Callback c);

    /** Runs the specified parameterless function <Callback> every <delay> milliseconds.
        The <counter> parameter is used for book-keeping and is updated on every call.
    **/
    static void everyMillis(const uint32_t delay, uint32_t &counter, Callback c);

  private:
    
};

#endif // PIMIENTO_SCHEDULER_H
